@@ -24,11 +24,11 @@
 
 #include <cstring>
 
-#include <QList>
-#include <QMap>
-
+#include <QColorSpace>
 #include <QDebug>
 #include <QImage>
+#include <QList>
+#include <QMap>
 
 class RLEData : public QList<uchar>
 {
@@ -225,7 +225,7 @@ bool SGIImagePrivate::readData(QImage &img)
         if (!getRow(line)) {
             return false;
         }
-        c = (QRgb *)img.scanLine(_ysize - y - 1);
+        c = reinterpret_cast<QRgb *>(img.scanLine(_ysize - y - 1));
         for (x = 0; x < _xsize; x++, c++) {
             *c = qRgb(line[x], line[x], line[x]);
         }
@@ -243,7 +243,7 @@ bool SGIImagePrivate::readData(QImage &img)
             if (!getRow(line)) {
                 return false;
             }
-            c = (QRgb *)img.scanLine(_ysize - y - 1);
+            c = reinterpret_cast<QRgb *>(img.scanLine(_ysize - y - 1));
             for (x = 0; x < _xsize; x++, c++) {
                 *c = qRgb(qRed(*c), line[x], line[x]);
             }
@@ -256,7 +256,7 @@ bool SGIImagePrivate::readData(QImage &img)
             if (!getRow(line)) {
                 return false;
             }
-            c = (QRgb *)img.scanLine(_ysize - y - 1);
+            c = reinterpret_cast<QRgb *>(img.scanLine(_ysize - y - 1));
             for (x = 0; x < _xsize; x++, c++) {
                 *c = qRgb(qRed(*c), qGreen(*c), line[x]);
             }
@@ -274,7 +274,7 @@ bool SGIImagePrivate::readData(QImage &img)
         if (!getRow(line)) {
             return false;
         }
-        c = (QRgb *)img.scanLine(_ysize - y - 1);
+        c = reinterpret_cast<QRgb *>(img.scanLine(_ysize - y - 1));
         for (x = 0; x < _xsize; x++, c++) {
             *c = qRgba(qRed(*c), qGreen(*c), qBlue(*c), line[x]);
         }
@@ -316,6 +316,9 @@ bool SGIImagePrivate::readImage(QImage &img)
         for (l = 0; !_stream.atEnd() && l < _numrows; l++) {
             _stream >> _starttab[l];
             _starttab[l] -= 512 + _numrows * 2 * sizeof(quint32);
+            if (_stream.status() != QDataStream::Ok) {
+                return false;
+            }
         }
         for (; l < _numrows; l++) {
             _starttab[l] = 0;
@@ -324,6 +327,9 @@ bool SGIImagePrivate::readImage(QImage &img)
         _lengthtab = new quint32[_numrows];
         for (l = 0; !_stream.atEnd() && l < _numrows; l++) {
             _stream >> _lengthtab[l];
+            if (_stream.status() != QDataStream::Ok) {
+                return false;
+            }
         }
     }
 
@@ -744,7 +750,14 @@ bool SGIImagePrivate::writeImage(const QImage &image)
         _dim = 3, _zsize++;
     }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+    auto cs = image.colorSpace();
+    if (cs.isValid() && cs.colorModel() == QColorSpace::ColorModel::Cmyk && image.format() == QImage::Format_CMYK8888) {
+        img = image.convertedToColorSpace(QColorSpace(QColorSpace::SRgb), QImage::Format_RGB32);
+    } else if (hasAlpha && img.format() != QImage::Format_ARGB32) {
+#else
     if (hasAlpha && img.format() != QImage::Format_ARGB32) {
+#endif
         img = img.convertToFormat(QImage::Format_ARGB32);
     } else if (!hasAlpha && img.format() != QImage::Format_RGB32) {
         img = img.convertToFormat(QImage::Format_RGB32);
